import { AllowedComponentProps } from 'vue';
import { ComponentCustomProps } from 'vue';
import { ComponentObjectPropsOptions } from 'vue';
import { ComponentOptions } from 'vue';
import { ComponentPublicInstance } from 'vue';
import { ExtractPropTypes } from 'vue';
import { UnwrapRef } from 'vue';
import { VNode } from 'vue';
import { VNodeProps } from 'vue';

export declare interface ClassComponentHooks {
    data?(): object;
    beforeCreate?(): void;
    created?(): void;
    beforeMount?(): void;
    mounted?(): void;
    beforeUnmount?(): void;
    unmounted?(): void;
    beforeUpdate?(): void;
    updated?(): void;
    activated?(): void;
    deactivated?(): void;
    render?(): VNode | void;
    errorCaptured?(err: Error, vm: Vue, info: string): boolean | undefined;
    serverPrefetch?(): Promise<unknown>;
}

export declare function createDecorator(factory: (options: ComponentOptions, key: string, index: number) => void): VueDecorator;

export declare function emits<EmitNames extends string>(emitNames: EmitNames[]): VueConstructor<Vue<unknown, EmitNames[]>>;

export declare function emits<EmitsOptions extends ObjectEmitsOptions>(emitsOptions: EmitsOptions): VueConstructor<Vue<unknown, EmitsOptions>>;

export declare type EmitsOptions = ObjectEmitsOptions | string[];

export declare type ExtractInstance<T> = T extends VueMixin<infer V> ? V : never;

export declare type MixedVueBase<Mixins extends VueMixin[]> = Mixins extends (infer T)[] ? VueConstructor<NarrowEmit<UnionToIntersection<ExtractInstance<T>> & Vue> & VueBase> : never;

export declare function mixins<T extends VueMixin[]>(...Ctors: T): MixedVueBase<T>;

export declare type NarrowEmit<T extends VueBase> = Omit<T, '$emit' | keyof ClassComponentHooks> & ClassComponentHooks & {
    $emit: T['$emit'] extends ((event: string, ...args: any[]) => void) & infer R ? unknown extends R ? T['$emit'] : R : T['$emit'];
};

export declare type ObjectEmitsOptions = Record<string, ((...args: any[]) => any) | null>;

export declare function Options<V extends Vue>(options: ComponentOptions & ThisType<V>): <VC extends VueConstructor>(target: VC) => VC;

export declare function props<PropNames extends string, Props = Readonly<{
    [key in PropNames]?: any;
}>>(propNames: PropNames[]): VueConstructor<Vue<Props, {}, VNodeProps & AllowedComponentProps & ComponentCustomProps> & Props>;

export declare function props<PropsOptions extends ComponentObjectPropsOptions>(propsOptions: PropsOptions): VueConstructor<Vue<Readonly<ExtractPropTypes<PropsOptions, false>>, {}, VNodeProps & AllowedComponentProps & ComponentCustomProps> & Readonly<ExtractPropTypes<PropsOptions>>>;

export declare function setup<R>(setupFn: () => R): UnwrapRef<R>;

export declare type UnionToIntersection<U> = (U extends any ? (k: U) => void : never) extends (k: infer I) => void ? I : never;

export declare type Vue<Props = unknown, Emits extends EmitsOptions = {}, PublicProps = Props & VNodeProps & AllowedComponentProps & ComponentCustomProps> = ComponentPublicInstance<Props, {}, {}, {}, {}, Emits, PublicProps> & ClassComponentHooks;

export declare const Vue: VueConstructor;

export declare type VueBase = Vue<unknown, never[]>;

export declare interface VueConstructor<V extends VueBase = Vue> extends VueMixin<V> {
    new (...args: any[]): V;
}

export declare interface VueDecorator {
    (Ctor: VueConstructor): void;
    (target: Vue, key: string): void;
    (target: Vue, key: string, index: number): void;
}

export declare type VueMixin<V extends VueBase = VueBase> = VueStatic & {
    prototype: V;
};

export declare interface VueStatic {
    /* Excluded from this release type: __vccCache */
    /* Excluded from this release type: __vccBase */
    /* Excluded from this release type: __vccDecorators */
    /* Excluded from this release type: __vccExtend */
    /* Excluded from this release type: __vccHooks */
    /* Excluded from this release type: __vccOpts */
    /* Excluded from this release type: render */
    /* Excluded from this release type: ssrRender */
    /* Excluded from this release type: __file */
    /* Excluded from this release type: __cssModules */
    /* Excluded from this release type: __scopeId */
    /* Excluded from this release type: __hmrId */
    registerHooks(keys: string[]): void;
}

export { }
